﻿Public Class frmRegen
    Dim lastTick, nextTick, finalTick As Date
    Dim dblCurrent, dblMax, dblRate As Double
    Dim blnCheckTickOnLoad, blnUpdateProgressBar, blnPercentMode As Boolean

    Private Sub frmRegen_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.Icon = My.Resources.icon_EHCalc_small
        Me.Location = My.Settings.powerRegenPosition

        numCurrent.Value = My.Settings.currentPower
        numRegen.Value = My.Settings.regenLevel
        dblRate = My.Settings.powerRegRate
        blnPercentMode = My.Settings.powerPercentMode
        blnCheckTickOnLoad = My.Settings.checkTick
        If blnCheckTickOnLoad Then
            numMax.Value = My.Settings.maxPower
            lastTick = My.Settings.lastTick
            nextTick = My.Settings.nextTick
            finalTick = My.Settings.finalTick
            LoadTime()
        Else
            numMax.Value = My.Settings.numPower
        End If
    End Sub

    Private Sub frmRegen_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        My.Settings.powerRegenPosition = Me.Location

        My.Settings.currentPower = numCurrent.Value
        If blnCheckTickOnLoad Then
            My.Settings.maxPower = numMax.Value
        Else
            My.Settings.numPower = numMax.Value
        End If
        My.Settings.regenLevel = numRegen.Value
        My.Settings.checkTick = blnCheckTickOnLoad
        My.Settings.nextTick = nextTick
        My.Settings.finalTick = finalTick
        My.Settings.powerRegRate = dblRate
        My.Settings.lastTick = lastTick
        My.Settings.powerPercentMode = blnPercentMode

        ' Have main window take focus
        frmMainInstance.Focus()
    End Sub

    Private Sub btnCalc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCalc.Click
        ' Calculate
        Dim dblRemaining, dblReg, dblPercentOfMax As Double
        Dim intTicks As Integer

        ' Grab inputs
        dblCurrent = CDbl(numCurrent.Value)
        dblMax = CDbl(numMax.Value)
        dblReg = CDbl(numRegen.Value)
        ' Sanity Check!
        If dblCurrent >= dblMax Then
            Timer.Enabled = False
            blnCheckTickOnLoad = False
            dblCurrent = dblMax
            numCurrent.Value = CDec(dblMax)
            txtTime.Text = "Full Charge"
            progressBar.Value = 100
            ' Why the hell should we continue?
            Return
        End If

        ' We will be updating the progress bar
        blnUpdateProgressBar = True

        ' What's our regen rate?
        dblPercentOfMax = dblMax * 0.0025
        If dblPercentOfMax > 0.5 Then
            dblRate = Math.Round(1 + (dblReg * dblPercentOfMax), MidpointRounding.AwayFromZero)
        Else
            dblRate = Math.Round(1 + (dblReg * 0.5), MidpointRounding.AwayFromZero)
        End If

        ' How much power do we need?
        dblRemaining = dblMax - dblCurrent

        ' Time to calculate Time. Server calculates per tick, which is every 10 minutes. Ex: 10:10pm, 10:20pm, etc
        ' When's next tick?
        GetNextTick()
        intTicks = CInt(Math.Floor(dblRemaining / dblRate) - 1)
        finalTick = nextTick.AddMinutes(intTicks * 10)
        lastTick = nextTick.AddMinutes(-10)
        Timer.Enabled = True
        UpdateTime()

    End Sub

    Private Sub UpdateTime()
        Dim dblPercent, dblMinutesTillNextTick, dblMinutesTillFinalTick As Double
        Dim timeTillNextTick, timeTillFinalTick As TimeSpan

        ' Compare now to next Tick
        timeTillNextTick = nextTick.Subtract(Now)
        dblMinutesTillNextTick = timeTillNextTick.TotalMinutes

        ' A tick has completed, update
        If dblMinutesTillNextTick <= 0 Then
            lastTick = nextTick
            nextTick = Now.AddMinutes(10)
            dblCurrent += dblRate
            If dblCurrent > dblMax Then
                dblCurrent = dblMax
            End If
            numCurrent.Value = CDec(dblCurrent)
        End If

        ' Update Progress Bar
        dblPercent = dblCurrent / dblMax
        progressBar.Value = CInt(Math.Floor(dblPercent * 100))
        ToolTips.SetToolTip(progressBar, "Regen Rate: " & dblRate.ToString())

        ' Calculate Time
        timeTillFinalTick = finalTick.Subtract(Now)
        dblMinutesTillFinalTick = timeTillFinalTick.TotalMinutes

        ' If time's up, stop timer
        If dblMinutesTillFinalTick <= 0 Then
            Timer.Enabled = False
            blnCheckTickOnLoad = False
            txtTime.Text = "Fully Charged"
            progressBar.Value = 100
            numCurrent.Value = CDec(dblMax)
            ToolTips.SetToolTip(progressBar, "100% Complete")
        Else
            blnCheckTickOnLoad = True
            If dblMinutesTillFinalTick > 60 Then
                Dim intHours As Double = CLng(dblMinutesTillFinalTick) \ 60
                dblMinutesTillFinalTick = dblMinutesTillFinalTick - (intHours * 60)
                txtTime.Text = intHours.ToString("0") & " Hour(s), " & dblMinutesTillFinalTick.ToString("0.0") & " Minute(s)"
            Else
                txtTime.Text = dblMinutesTillFinalTick.ToString("0.#") & " Minute(s)"
            End If
        End If

    End Sub

    Private Sub LoadTime()
        ' We're loading the program, find out how long we've been down and update time accordingly
        Dim timeLeft, timePast As TimeSpan
        Dim prevTick As Date
        Dim intTicks As Integer
        Dim dblMinutesLeft As Double

        ' Get Current Power
        dblCurrent = CDbl(numCurrent.Value)
        dblMax = CDbl(numMax.Value)

        ' Reset Progress Bar, just in case
        progressBar.Value = 0

        timeLeft = finalTick.Subtract(Now)
        dblMinutesLeft = timeLeft.TotalMinutes()

        If dblMinutesLeft <= 0 Then
            ' Power Fully restored.
            txtTime.Text = "Fully Charged"
            dblCurrent = dblMax
            numCurrent.Value = CDec(dblCurrent)
            progressBar.Value = 100
        Else
            ' Find out where we are in the charge cycle
            GetNextTick()
            prevTick = nextTick.AddMinutes(-10)
            timePast = prevTick.Subtract(lastTick)
            intTicks = CInt(Math.Floor(timePast.TotalMinutes() / 10))
            If intTicks < 1 Then
                intTicks = 0
            Else
                lastTick = prevTick
            End If
            dblCurrent += (intTicks * dblRate)
            ' Sanity Check
            If dblCurrent > dblMax Then
                dblCurrent = dblMax
            End If
            numCurrent.Value = CDec(dblCurrent)
            Timer.Enabled = True
            UpdateTime()
        End If

    End Sub

    Private Sub Timer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer.Tick
        UpdateTime()
    End Sub

    ' Returns the next server tick which is every 10 minutes (13:10, 13:20, etc)
    Private Sub GetNextTick()
        Dim intMinutesTillTick As Integer

        intMinutesTillTick = 10 - (Now.Minute() Mod 10)
        If intMinutesTillTick < 10 Then
            nextTick = Now.AddMinutes(intMinutesTillTick)
        Else
            intMinutesTillTick = 0
            nextTick = Now.AddMinutes(10)
        End If
        nextTick = nextTick.AddSeconds(-nextTick.Second)
    End Sub

    Private Sub HighlightNum(ByVal sender As Object, ByVal e As System.EventArgs) Handles numCurrent.Enter, numMax.Enter, _
    numRegen.Enter
        Dim newSender As Windows.Forms.NumericUpDown = CType(sender, Windows.Forms.NumericUpDown)
        newSender.Select(0, 6)
    End Sub

    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        Me.Close()
    End Sub
End Class