﻿' Main Window for EH Calculator; allows access to the rest of the program

Imports System.Xml

Public Class frmMain
    Dim dateNextHVBattle, dateNextHVNotify, datePowerRegen As Date
    Dim intHVNotifyMinutes As Integer = 15
    Dim intHVLaunchMode As Integer = -1
    Dim blnHVNotify As Boolean = False
    Dim blnPowerRegenNotify As Boolean = False
    Dim strHVLaunchURL As String = "http://e-hentai.org"

    ' Forms that should not be multi-instanced
    Private frmPowerRegen As New frmRegen

#Region "Main Form Routines"
    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Stuff we do on loading up the form. Set icon, set checkboxes, etc.
        Me.Icon = My.Resources.icon_EHCalc_small
        nfiMain.Icon = My.Resources.icon_EHCalc_small
        menuSaveSettings.Checked = mblnSaveSettings

        ' Do we check for updates?
        If My.Settings.autoUpdate Then
            Dim lastChecked As TimeSpan

            lastChecked = Date.Now.Subtract(My.Settings.lastUpdateCheck)
            If lastChecked.TotalDays >= My.Settings.autoUpdateDays Then
                Dim result As DialogResult
                result = MessageBox.Show("I am about to check for updates, is this ok?", "EH Calculator Update", MessageBoxButtons.YesNo, MessageBoxIcon.Question)
                If result = MsgBoxResult.Yes Then
                    CheckForUpdates()
                Else
                    MessageBox.Show("Not checking for updates.", "EH Calculator Update", MessageBoxButtons.OK, MessageBoxIcon.Information)
                End If
            End If
        End If

        ' Do we ask to backup the save file?
        If My.Settings.autoBackup Then
            Dim lastBackup As TimeSpan

            lastbackup = Date.Now.Subtract(My.Settings.lastBackup)
            If lastBackup.TotalDays >= My.Settings.autoBackupDays Then
                Dim result As DialogResult
                result = MessageBox.Show("Would you like to make a backup of your settings?", "EH Calculator Backup", _
                                         MessageBoxButtons.YesNo, MessageBoxIcon.Question)
                If result = MsgBoxResult.Yes Then
                    BackupSettings()
                End If
            End If
        End If

        ' Update the profile summary
        UpdateProfileSummary()

    End Sub

    ' This sub stops the form from closing if minimizeInstead was set and the user pushed the [x] button
    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If My.Settings.minimizeInstead And (e.CloseReason = CloseReason.UserClosing) Then
            e.Cancel = True
            Me.WindowState = FormWindowState.Minimized
        End If
    End Sub

    Private Sub frmMain_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Me.WindowState = FormWindowState.Minimized Then
            Me.Hide()
            If My.Settings.firstMinimize Then
                ' This is the first time we minimized, let the user know we go to tray now
                My.Settings.firstMinimize = False
                nfiMain.ShowBalloonTip(10000, "EH Calculator", "The program has now minimized to the system tray. Click here to restore it.", ToolTipIcon.Info)
            End If
        End If
    End Sub
#End Region

#Region "Notification Tray"
    Private Sub nfiMain_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles nfiMain.MouseClick
        If e.Button = Windows.Forms.MouseButtons.Left Then
            Me.Visible = True
            Me.WindowState = FormWindowState.Normal
            Me.Focus()
        End If
    End Sub
#End Region

#Region "Menu Items"
    ' Contains all the click actions for the menu
    Private Sub menuSaveSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuSaveSettings.Click
        If menuSaveSettings.Checked Then
            mblnSaveSettings = True
        Else
            mblnSaveSettings = False
        End If
    End Sub

    ' Clear Saved Settings
    Private Sub menuClearSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuClearSettings.Click
        Dim result As DialogResult

        ' Confirm that we want to clear settings
        result = MessageBox.Show("Are you sure you want to clear all settings?", "EH Calculator", MessageBoxButtons.YesNo, MessageBoxIcon.Question)

        ' If yes, go ahead and clear
        If result = Windows.Forms.DialogResult.Yes Then
            System.IO.File.Delete(Application.StartupPath & "\settings.xml")
            mblnResetSettings = True
            Application.Restart()
        End If

    End Sub

    Private Sub menuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuExit.Click
        Application.Exit()
    End Sub

    Private Sub menuCheckforUpdates_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuCheckforUpdates.Click
        CheckForUpdates()
    End Sub

    Private Sub menuMyProfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuMyProfile.Click
        MyProfileOpen()
    End Sub

    Private Sub btnMyProfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMyProfile.Click
        MyProfileOpen()
    End Sub

    Private Sub MyProfileOpen()
        Dim frmMyProfile As New frmProfile
        ' Unlike other Dialogs, we do not want other stuff going on while this is open!
        frmMyProfile.ShowDialog()
        frmMyProfile = Nothing

        ' In case something changed...
        UpdateProfileSummary()
    End Sub

    Private Sub menuAdditionalOptionsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuAdditionalOptionsToolStripMenuItem.Click
        Dim frmOptions As New frmOptions
        ' Just like the Profile dialog, we don't want other stuff going on.
        frmOptions.ShowDialog()
        frmOptions = Nothing
    End Sub

    Private Sub menuHVBattleTimer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuHVBattleTimer.Click
        Dim frmHVBattleTimer As New frmHVBattleTimer
        With frmHVBattleTimer
            .launchMode = intHVLaunchMode
            .NotifyEnabled = blnHVNotify
            .NotifyMinutes = intHVNotifyMinutes
            .ShowDialog()
        End With

        If Not frmHVBattleTimer.Canceled Then
            ' User didn't cancel, so what do we do?
            If frmHVBattleTimer.EnableTimer Then
                ' Setup Timer and start it
                dateNextHVBattle = frmHVBattleTimer.NextBattle
                intHVLaunchMode = frmHVBattleTimer.launchMode
                blnHVNotify = frmHVBattleTimer.NotifyEnabled
                intHVNotifyMinutes = frmHVBattleTimer.NotifyMinutes
                dateNextHVNotify = Date.Now.AddMinutes(intHVNotifyMinutes)
                StartTimer()
            Else
                ' Abort the Timer
                AbortTimer()
            End If
        End If

        frmHVBattleTimer = Nothing
    End Sub

    Private Sub menuRefreshSummary_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuRefreshSummary.Click
        UpdateProfileSummary()
    End Sub

    Private Sub menuBackupSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuBackupSettings.Click
        BackupSettings()
    End Sub

    Private Sub menuRecoverSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuRecoverSettings.Click
        RestoreSettings()
    End Sub

    Private Sub menuSaveNow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuSaveNow.Click
        SaveSettings()
    End Sub

    Private Sub menuDocumentation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuDocumentation.Click
        MessageBox.Show("Have not written one yet.", "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Sub
#End Region

#Region "Toggle System Menu"
    ' Toggle System Menu clicks; what you get when you right click the notification tray icon
    Private Sub tsmExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmExit.Click
        Application.Exit()
    End Sub

    Private Sub tsmStopTimer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmStopTimer.Click
        AbortTimer()
    End Sub

    Private Sub tsmRestartTimer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmRestartTimer.Click
        ' Safe guard, only do this if timer is already up!
        If HVTimer.Enabled Then
            dateNextHVBattle = Date.Now.AddHours(1.0)
            dateNextHVNotify = Date.Now.AddMinutes(intHVNotifyMinutes)
        End If
    End Sub

    Private Sub tsmShowProgram_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmShowProgram.Click
        Me.Visible = True
        Me.WindowState = FormWindowState.Normal
        Me.Focus()
    End Sub
#End Region

#Region "Tool Buttons"
    ' Contains all the click actions for the Tool Buttons
    Private Sub btnGPcalc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGPcalc.Click
        Dim frmGPTool As New frmGPTool
        frmGPTool.Show()
    End Sub

    Private Sub btnKarma_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnKarma.Click
        Dim frmKarma As New frmKarma
        frmKarma.Show()
    End Sub

    Private Sub btnTraining_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTraining.Click
        Dim frmTraining As New frmTraining
        frmTraining.Show()
    End Sub

    Private Sub menuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuAbout.Click
        Dim frmAbout As New frmSplash
        With frmAbout
            .Splash = False
            .ShowDialog()
        End With
    End Sub

    Private Sub btnHathCalc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnHathCalc.Click
        Dim frmHath As New frmHath
        frmHath.Show()
    End Sub

    Private Sub btnRegen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRegen.Click
        If Not frmPowerRegen.Created Then
            frmPowerRegen = New frmRegen
            frmPowerRegen.Show()
        Else
            frmPowerRegen.Focus()
        End If
    End Sub

    Private Sub btnEHTrack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEHTrack.Click
        Dim frmEHTracker As New frmEHTrack
        frmEHTracker.Show()
    End Sub

    Private Sub btnHVTool_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnHVTool.Click
        Dim frmHVStat As New frmHVStat
        frmHVStat.Show()
    End Sub

    Private Sub btnEXPCalc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEXPCalc.Click
        Dim frmEXPCalc As New frmEXP
        frmEXPCalc.Show()
    End Sub
#End Region

#Region "HV Timer Stuff"
    'Stuff for the HV Timer
    Private Sub HVTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HVTimer.Tick
        Dim tsNextBattle As TimeSpan
        Dim tsNextNotfiy As TimeSpan
        Dim strTimeRemaining As String

        tsNextBattle = dateNextHVBattle.Subtract(Date.Now)
        tsNextNotfiy = dateNextHVNotify.Subtract(Date.Now)

        Dim intHours As Integer = tsNextBattle.Hours
        Dim intTotalMinutes As Integer = CInt(Math.Ceiling(tsNextBattle.TotalMinutes))
        Dim intMinutes As Integer = tsNextBattle.Minutes
        Dim intSeconds As Integer = tsNextBattle.Seconds

        If intHours > 0 Then
            strTimeRemaining = intHours.ToString & "h " & intMinutes.ToString & "m " & intSeconds.ToString & "s"
        Else
            strTimeRemaining = intMinutes.ToString & "m " & intSeconds.ToString & "s"
        End If

        lblHVTimer.Text = strTimeRemaining
        nfiMain.Text = "EH Calculator" & vbCrLf & "Next HV Battle in: " & strTimeRemaining

        If tsNextBattle.TotalSeconds <= 0 Then
            ' Time's up, lets do the launch
            ' Stop Timer to prevent multiple launches
            HVTimer.Stop()
            If intHVLaunchMode = BROWSERLAUNCH Then
                ' Do we use the system default web brwoser?
                If My.Settings.customBrowser Then
                    Dim p As New ProcessStartInfo
                    p.FileName = My.Settings.browserPath
                    p.Arguments = strHVLaunchURL
                    Process.Start(p)
                Else
                    Process.Start(strHVLaunchURL)
                End If
                nfiMain.ShowBalloonTip(5000, "HV Battle Alert!", "It is time for your hourly HV Battle!", ToolTipIcon.Info)
                ' Add 0.5 minutes to accomidate internet lag
                dateNextHVBattle = Date.Now.AddMinutes(60.5)
                dateNextHVNotify = Date.Now.AddMinutes(intHVNotifyMinutes + 0.5)
            Else
                ' User Oppted only to be notify by Balloon
                nfiMain.ShowBalloonTip(20000, "HV Battle Alert!", "It is time for your hourly HV Battle!", ToolTipIcon.Info)
                dateNextHVBattle = Date.Now.AddMinutes(62)
                dateNextHVNotify = Date.Now.AddMinutes(intHVNotifyMinutes + 2)
            End If
            HVTimer.Start()
        ElseIf blnHVNotify Then
            ' Do we Notify Time Remaining?
            If tsNextNotfiy.TotalSeconds <= 0 Then
                If intTotalMinutes <= 1 Then
                    nfiMain.ShowBalloonTip(10000, "HV Battle Timer", intTotalMinutes.ToString & " minute till next Battle", ToolTipIcon.None)
                Else
                    nfiMain.ShowBalloonTip(10000, "HV Battle Timer", intTotalMinutes.ToString & " minutes till next Battle", ToolTipIcon.None)
                End If
                dateNextHVNotify = Date.Now.AddMinutes(intHVNotifyMinutes)
            End If
        End If

    End Sub

    ' Starts the HV Timer
    Private Sub StartTimer()
        HVTimer.Enabled = True
        HVTimer.Start()
        lblRandomEncounterLabel.Visible = True
        lblHVTimer.Visible = True
    End Sub

    ' Stop the HV Timer and close it
    Private Sub AbortTimer()
        HVTimer.Stop()
        HVTimer.Enabled = False
        lblRandomEncounterLabel.Visible = False
        lblHVTimer.Visible = False
        Me.Visible = True
        Me.WindowState = FormWindowState.Normal
    End Sub
#End Region

#Region "Power Regen Calc Stuff"
    ' Stuff for the Power Regen Notifcation Timer

    ' Setups the timer to notify when Power Regen is done
    Public Sub checkForPowerRegen(ByVal activate As Boolean, ByVal powerRegenDate As Date)
        If activate Then
            blnPowerRegenNotify = True
            datePowerRegen = powerRegenDate
            PowerRegenTimer.Enabled = True
            PowerRegenTimer.Start()
        Else
            blnPowerRegenNotify = False
            PowerRegenTimer.Stop()
            PowerRegenTimer.Enabled = False
        End If
    End Sub

    Private Sub PowerRegenTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PowerRegenTimer.Tick
        ' We shouldn't be here if we aren't looking, but just incase
        If blnPowerRegenNotify = False Then
            Exit Sub
        End If

        Dim tsTimeLeft As TimeSpan
        tsTimeLeft = datePowerRegen.Subtract(Now)

        If tsTimeLeft.TotalSeconds < 1 Then
            nfiMain.ShowBalloonTip(10000, "EH Calc: Power Regen", "Your power on e-hentai.org is now fully recharged.", ToolTipIcon.Info)
            blnPowerRegenNotify = False
            PowerRegenTimer.Stop()
            PowerRegenTimer.Enabled = False
        End If
    End Sub
#End Region

#Region "Bonus Calculation Stuff"
    ' Called to update the profile summary section
    Private Sub UpdateProfileSummary()
        ' Did user setup their profile?
        Dim tsDefaultSpan As TimeSpan
        Dim dateDefault As Date = Date.Parse("1/1/2005")
        Dim tsDaysReg As TimeSpan

        ' Don't bother calculating days registered if they haven't told the calc when they
        ' registered.
        tsDefaultSpan = My.Settings.dateReg.Subtract(dateDefault)
        If tsDefaultSpan.TotalHours < 1 Then
            lblDaysRegistered.Text = "N/A"
            ToolTips.SetToolTip(lblDaysRegistered, "Set Date Registered in Profile Please")
            Exit Sub
        Else
            tsDaysReg = Now.Subtract(My.Settings.dateReg)
            lblDaysRegistered.Text = Math.Floor(tsDaysReg.TotalDays).ToString("#,#")
            ToolTips.SetToolTip(lblDaysRegistered, tsDaysReg.TotalDays.ToString("#,#.#"))
        End If

        ' If we got here, they have setup their profile
        Dim dblAuraExp, dblExpTotal, dblDawnTotal, dblPowerBase, dblAuraPower, dblDawn, _
        dblCreditPerkMod, dblExpPerkMod, dblExpToplist, dblExpPosts, dblMiscPost As Double
        Dim intPosts As Integer

        dblMiscPost = MISCPOST_BASE

        ' Add up Aura's
        dblAuraExp = 1
        If My.Settings.redAuraChecked Then
            dblAuraExp += EXPMOD_REDAURA
            dblAuraPower += PMOD_REDAURA
        End If
        If My.Settings.orangeAuraChecked Then
            dblAuraExp += EXPMOD_ORANGEAURA
            dblAuraPower += PMOD_ORANGEAURA
        End If
        If My.Settings.yellowAuraChecked Then
            dblAuraExp += EXPMOD_YELLOWAURA
            dblAuraPower += PMOD_YELLOWAURA
        End If
        If My.Settings.greenAuraChecked Then
            dblAuraExp += EXPMOD_GREENAURA
            dblAuraPower += PMOD_GREENAURA
        End If
        If My.Settings.blueAuraChecked Then
            dblAuraExp += EXPMOD_BLUEAURA
            dblAuraPower += PMOD_BLUEAURA
        End If
        If My.Settings.indigoAuraChecked Then
            dblAuraExp += EXPMOD_INDIGOAURA
            dblAuraPower += PMOD_INDIGOAURA
        End If
        If My.Settings.violetAuraChecked Then
            dblAuraExp += EXPMOD_VIOLETAURA
            dblAuraPower += PMOD_VIOLETAURA
        End If
        If My.Settings.blackAuraChecked Then
            dblAuraExp += EXPMOD_BLACKAURA
            dblAuraPower += PMOD_BLACKAURA
        End If
        If My.Settings.whiteAuraChecked Then
            dblAuraExp += EXPMOD_WHITEAURA
            dblAuraPower += PMOD_WHITEAURA
        End If
        If My.Settings.rainbowAuraChecked Then
            dblAuraExp += EXPMOD_RAINBOWAURA
            dblAuraPower += PMOD_RAINBOWAURA
        End If
        If My.Settings.bronzeAuraChecked Then
            dblAuraExp += EXPMOD_BRONZEAURA
            dblAuraPower += PMOD_BRONZEAURA
            dblDawn = DAWN_BRONZE
            dblMiscPost += MISCPOST_BRONZE
        End If
        If My.Settings.silverAuraChecked Then
            dblAuraExp += EXPMOD_SILVERAURA
            dblAuraPower += PMOD_SILVERAURA
            dblDawn = DAWN_SILVER
            dblMiscPost += MISCPOST_SILVER
        End If
        If My.Settings.goldAuraChecked Then
            dblAuraExp += EXPMOD_GOLDAURA
            dblAuraPower += PMOD_GOLDAURA
            dblDawn = DAWN_GOLD
            dblMiscPost += MISCPOST_GOLD
        End If
        If My.Settings.triAuraChecked Then
            dblAuraExp += EXPMOD_TRIAURA
            dblAuraPower += PMOD_TRIAURA
            dblDawn = DAWN_TRI
            dblMiscPost += MISCPOST_TRI
        End If
        If My.Settings.quintAuraChecked Then
            dblAuraExp += EXPMOD_QUINTAURA
            dblAuraPower += PMOD_QUINTAURA
            dblDawn = DAWN_QUINT
            dblMiscPost += MISCPOST_QUINT
        End If
        If My.Settings.septuaAuraChecked Then
            dblAuraExp += EXPMOD_SEPTUAAURA
            dblAuraPower += PMOD_SEPTUAAURA
            dblDawn = DAWN_SEPTUA
            dblMiscPost += MISCPOST_SEPTUA
        End If
        If My.Settings.catgirlAuraChecked Then
            dblAuraExp += EXPMOD_CATGIRLAURA
            dblAuraPower += PMOD_CATGIRLAURA
            dblDawn = DAWN_CATGIRL
            dblMiscPost += MISCPOST_CATGIRL
        End If
        If My.Settings.awardCookie Then
            dblAuraExp += EXPMOD_COOKIE
            dblAuraPower += PMOD_COOKIE
            dblDawn += DAWN_COOKIE
        End If
        If My.Settings.awardTwin Then
            dblAuraExp += EXPMOD_TWIN
            dblAuraPower += PMOD_TWIN
            dblDawn += DAWN_TWIN
        End If
        If My.Settings.awardDull Then
            dblAuraExp += EXPMOD_DULL
            dblAuraPower += PMOD_DULL
            dblDawn += DAWN_DULL
        End If
        If My.Settings.awardGoldenAt Then
            dblAuraExp += EXPMOD_GOLDENAT
        End If
        If My.Settings.awardYellowCrystal Then
            dblAuraExp += EXPMOD_YELLOWCRYSTAL
            dblAuraPower += PMOD_YELLOWCRYSTAL
            dblDawn += DAWN_YELLOWCRYSTAL
        End If
        If My.Settings.awardBlueCrystal Then
            dblAuraExp += EXPMOD_BLUECRYSTAL
            dblAuraPower += PMOD_BLUECRYSTAL
            dblDawn += DAWN_BLUECRYSTAL
        End If
        If My.Settings.awardPurpleCrystal Then
            dblAuraExp += EXPMOD_PURPLECRYSTAL
            dblAuraPower += PMOD_PURPLECRYSTAL
            dblDawn += DAWN_PURPLECRYSTAL
        End If
        If My.Settings.awardHand Then
            dblAuraExp += EXPMOD_SNOWFLAKEHAND
            dblDawn += DAWN_SNOWFLAKEHAND
        End If
        If My.Settings.awardPride Then
            dblAuraExp += EXPMOD_SNOWFLAKEPRIDE
            dblDawn += DAWN_SNOWFLAKEPRIDE
        End If
        If My.Settings.awardEsteem Then
            dblAuraExp += EXPMOD_SNOWFLAKEESTEEM
            dblDawn += DAWN_SNOWFLAKEESTEEM
        End If

        ' Dawn of a new day for a regular user, 5 to 15 C or so.
        If dblDawn < DAWN_BRONZE Then
            dblDawn = DAWN_NONE
        End If

        ' What Perk do we have that modifies EXP?
        dblExpPerkMod = 1
        Select Case (My.Settings.expPerk)
            Case MODE_ORGANIZEDMIND
                dblExpPerkMod += EXPMOD_ORGANIZEDMIND
            Case MODE_DISCIPLINEDLEARNER
                dblExpPerkMod += EXPMOD_DISCIPLINEDLEARNER
            Case MODE_THINKINGCAP
                dblExpPerkMod += EXPMOD_THINKINGCAP
            Case MODE_MENTATS
                dblExpPerkMod += EXPMOD_MENTATS
            Case MODE_LEARNINGCHIP
                dblExpPerkMod += EXPMOD_LEARNINGCHIP
            Case MODE_CYBERNETICBRAIN
                dblExpPerkMod += EXPMOD_CYBERNETICBRAIN
        End Select

        ' Lets look at Toplist EXP
        ' Use the EXP_TOPLIST array to figure out bonus
        dblExpToplist = EXP_TOPLIST(CInt(My.Settings.toplistGallery)) + EXP_TOPLIST(CInt(My.Settings.toplistUploader)) _
        + EXP_TOPLIST(CInt(My.Settings.toplistTagging)) + EXP_TOPLIST(CInt(My.Settings.toplistHath)) + _
        EXP_TOPLIST(CInt(My.Settings.toplistEHTracker)) + EXP_TOPLIST(CInt(My.Settings.toplistCleanup)) + _
        EXP_TOPLIST(CInt(My.Settings.toplistReviewer))

        ' Avg Posts
        intPosts = CInt(My.Settings.numAvgPost)
        If intPosts > 0 Then
            ' Calculation given by Tenboro
            dblExpPosts = 50 * Math.Log10(intPosts)
            ' Bonus Capped at 100%
            If dblExpPosts > 100 Then
                dblExpPosts = 100
            End If
        Else
            ' We have diddly squat!
            dblExpPosts = 0
        End If

        ' Calculate EXP Bonus
        dblExpTotal = dblAuraExp * (1 + (My.Settings.expSkills * EXPMOD_EXPSKILL) + (My.Settings.adeptLearner * EXPMOD_ADEPTLEARNER) + (dblExpPosts / 100) + dblExpToplist) * dblExpPerkMod
        dblExpTotal -= 1

        ' What Perk do we have that modifies Credit?
        dblCreditPerkMod = 1
        Select Case (My.Settings.creditPerk)
            Case MODE_PENNYPINCHER
                dblCreditPerkMod += CMOD_PENNYPINCHER
            Case MODE_FRUGALITY
                dblCreditPerkMod += CMOD_FRUGALITY
            Case MODE_MASTERMISER
                dblCreditPerkMod += CMOD_MASTERMISER
            Case MODE_STOCKOPTIONS
                dblCreditPerkMod += CMOD_STOCKOPTIONS
            Case MODE_SHORTSELLER
                dblCreditPerkMod += CMOD_SHORTSELLER
            Case MODE_GOLDENHANDSHAKE
                dblCreditPerkMod += CMOD_GOLDENHANDSHAKE
        End Select

        ' Calculate Credit and Dawn of a New Day Bonus
        ' I don't think the server is rounding DOWN anymore
        dblDawnTotal = Math.Round(dblDawn * dblCreditPerkMod)

        ' Subtract by one so the % display shows 1%, 8%, etc and not 101%, 108%
        dblCreditPerkMod -= 1.0

        ' Calculate Power Tanks
        dblPowerBase = My.Settings.powerTank / 2

        ' Calculate Misc Posting Total!
        dblMiscPost += (My.Settings.miscPosting * MISCPOST_TRAINING) + (tsDaysReg.TotalDays * MISCPOST_DAY)

        ' Update Display
        lblLvl.Text = My.Settings.henjutsuLevel.ToString()
        lblEXPBonus.Text = dblExpTotal.ToString("0.#%")
        lblCreditBonus.Text = dblCreditPerkMod.ToString("0.#%")
        lblPowerBaseBonus.Text = "+ " & dblPowerBase.ToString("0.#")
        lblPowerBonus.Text = dblAuraPower.ToString("0.#%")
        lblDOAND.Text = dblDawnTotal.ToString("#,#") & " C"
        lblMiscLimit.Text = dblMiscPost.ToString("0.##")
    End Sub
#End Region

#Region "Backup and Restore Settings Stuff"
    Sub BackupSettings()
        Dim strBackupFile, strSettingsFile As String
        Dim result As DialogResult

        ' If the save dialog hasn't been used during the current run time, set default save loction
        ' to My Documents
        If saveFD.InitialDirectory.Length < 2 Then
            saveFD.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)
        End If

        ' Create a default save name if needed
        If saveFD.FileName.Length < 2 Then
            saveFD.FileName = "EH Calc v3 Backup " & Date.Now.ToString("yyyy\-MM\-dd\ HHmm")
        End If


        ' Ask where to save the file
        result = saveFD.ShowDialog()

        ' Did they hit the cancel button?
        If Not result = Windows.Forms.DialogResult.Cancel Then
            ' Just in case
            If saveFD.FileName.Length > 2 Then
                ' Save Settings before we back them up!
                My.Settings.Save()

                ' Get save file
                strBackupFile = saveFD.FileName

                ' Where are we running from Currently?
                strSettingsFile = Application.StartupPath & "\settings.xml"

                ' Copy settings file to new location
                If System.IO.File.Exists(strSettingsFile) Then
                    ' Just in case user tries to save to read-only place or something odd happens
                    Try
                        System.IO.File.Copy(strSettingsFile, strBackupFile, True)
                    Catch ex As Exception
                        MessageBox.Show("There was an error backing up settings." & vbNewLine & ex.Message, _
                                        "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    End Try
                    My.Settings.lastBackup = Now
                Else
                    MessageBox.Show("I was not able to backup settings as there were no settings to backup.", _
                                    "EH calculator", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                End If
            Else
                MessageBox.Show("Bad File Name, aborting", "EH calculator", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If
        End If
    End Sub

    Sub RestoreSettings()
        Dim strBackupFile, strSettingsFile As String
        Dim result As DialogResult

        If openFD.InitialDirectory.Length < 2 Then
            openFD.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)
        End If

        ' Ask for the backup file
        result = openFD.ShowDialog

        ' Did they hit the cancel button?
        If Not result = Windows.Forms.DialogResult.Cancel Then
            ' Just in case
            strBackupFile = openFD.FileName

            ' Where ae we currently running from?
            strSettingsFile = Application.StartupPath & "\settings.xml"

            ' Validate Backup File!
            If System.IO.File.Exists(strBackupFile) Then
                ' Use XMLDocument to read backup file first to validate it
                Dim xmlBackupFile As XmlDocument = New XmlDocument()
                Dim backupFileNode As XmlNode
                Dim strFileTest As String

                ' Load strBackupFile into XMLDocument and attempt to read the following Node
                xmlBackupFile.Load(strBackupFile)
                backupFileNode = xmlBackupFile.SelectSingleNode("EHCalc/isEHCalcv3SettingFile")

                If backupFileNode Is Nothing Then 'File does not appear to be valid, abort!
                    MessageBox.Show("The file you specified does not appear to be a valid EH Calculator v3 settings file.", _
                "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Exit Sub
                Else
                    strFileTest = backupFileNode.InnerText.Trim
                    If strFileTest.Length < 1 Or (Not strFileTest.Contains("True")) Then 'File does not appear to be valid, abort!
                        MessageBox.Show("The file you specified does not appear to be a valid EH Calculator v3 settings file.", _
                "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
                        Exit Sub
                    End If
                End If

                ' File appears legit, lets copy it into our settings
                Try
                    System.IO.File.Copy(strBackupFile, strSettingsFile, True)
                Catch ex As Exception
                    MessageBox.Show("There was an error restoring settings." & vbNewLine & ex.Message, _
                                    "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
                End Try

                ' Restart application to flush out all persistant data
                mblnResetSettings = True
                Application.Restart()

            Else
                MessageBox.Show("No such file exists!", "EH Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        End If
    End Sub
#End Region

End Class