﻿' mUpdate Module
' Checks for EH Calculator v3 updates by downloading an XML from a web server
' and compares current version to the one downloaded.
' If you make changes to this program and change the version, be sure to take
' into account the default XMLURL!

Imports System.Xml

Module mUpdate
    ' Where do we go to get the current version info?
    Const XMLURL As String = "http://www.starfleetplatoon.com/EHCalc/update.xml"

    Public Sub CheckForUpdates()
        Dim strCurrentVersion As String = "" ' Used for Current Version
        Dim strUpdateURL As String = "" ' Used for where we send user to get update if available

        ' Grab relevent information from XML
        Using reader As XmlTextReader = New XmlTextReader(XMLURL)
            While (reader.Read())
                If (reader.NodeType = XmlNodeType.Element And "Version" = reader.Name) Then
                    strCurrentVersion = reader.ReadElementString("Version")
                End If
                If (reader.NodeType = XmlNodeType.Element And "URL" = reader.Name) Then
                    strUpdateURL = reader.ReadElementString("URL")
                End If
            End While
        End Using

        ' If XML is curropt or we didn't get one in the first place, abort
        If String.IsNullOrEmpty(strCurrentVersion) Then
            MessageBox.Show("Could not contact update server.", "EH Calculator Update", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' If we got this far, we can say we've successfully checked; update last checked date!
        My.Settings.lastUpdateCheck = Date.Now

        ' What are we currently Running?
        Dim currentVersion As New System.Version(strCurrentVersion)
        If My.Application.Info.Version.CompareTo(currentVersion) < 0 Then
            ' Newer version detected, ask user if they want to go to the Download Page
            Dim result As DialogResult
            result = MessageBox.Show("A new update has been found. Proceed to the Download Page?", "EH Calculator Update", MessageBoxButtons.YesNo, MessageBoxIcon.Question)
            If result = DialogResult.Yes Then
                openWebBrowser(strUpdateURL)
            End If
        Else
            ' We apparently have the latest version, let user know and move on.
            MessageBox.Show("You have the most up-to-date version.", "EH Calculator Update", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

End Module
